@@ -25,46 +25,45 @@ and its expressions, changing the meaning of course dramatically.
 from nuitka import Variables, Builtins
 
 from .NodeBases import (
-    StatementChildrenHavingBase,
     ExpressionMixin,
     NodeBase
 )
 
 from .ConstantRefNodes import ExpressionConstantRef
 
-def _isReadOnlyUnterdeterminedModuleVariable( variable ):
+def _isReadOnlyUnterdeterminedModuleVariable(variable):
     return variable.isModuleVariable() and \
            variable.getReadOnlyIndicator() is None
 
-def _isReadOnlyModuleVariable( variable ):
+def _isReadOnlyModuleVariable(variable):
     return (
         variable.isModuleVariable() and \
         variable.getReadOnlyIndicator() is True
     ) or variable.isMaybeLocalVariable()
 
 
-class ExpressionVariableRef( NodeBase, ExpressionMixin ):
+class ExpressionVariableRef(NodeBase, ExpressionMixin):
     kind = "EXPRESSION_VARIABLE_REF"
 
-    def __init__( self, variable_name, source_ref ):
+    def __init__(self, variable_name, source_ref):
         NodeBase.__init__( self, source_ref = source_ref )
 
         self.variable_name = variable_name
         self.variable = None
 
-    def getDetails( self ):
+    def getDetails(self):
         if self.variable is None:
             return { "name" : self.variable_name }
         else:
             return { "name" : self.variable_name, "variable" : self.variable }
 
-    def getDetail( self ):
+    def getDetail(self):
         if self.variable is None:
             return self.variable_name
         else:
             return repr( self.variable )
 
-    def makeCloneAt( self, source_ref ):
+    def makeCloneAt(self, source_ref):
         result = self.__class__(
             variable_name = self.variable_name,
             source_ref    = source_ref
@@ -74,27 +73,31 @@ class ExpressionVariableRef( NodeBase, ExpressionMixin ):
 
         return result
 
-    def isTargetVariableRef( self ):
+    def isTargetVariableRef(self):
         return False
 
-    def getVariableName( self ):
+    def getVariableName(self):
         return self.variable_name
 
-    def getVariable( self ):
+    def getVariable(self):
         return self.variable
 
-    def setVariable( self, variable ):
+    def setVariable(self, variable):
         assert isinstance( variable, Variables.Variable ), repr( variable )
         assert self.variable is None
 
         self.variable = variable
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
         assert self.variable is not None
 
         if _isReadOnlyUnterdeterminedModuleVariable( self.variable ):
             constraint_collection.assumeUnclearLocals()
-            constraint_collection.signalChange( "new_expression", self.source_ref, "txt" )
+            constraint_collection.signalChange(
+                "new_expression",
+                self.source_ref,
+                "Unclear module variable delays processing."
+            )
 
         if _isReadOnlyModuleVariable( self.variable ):
             if self.variable_name in Builtins.builtin_exception_names:
@@ -107,7 +110,10 @@ class ExpressionVariableRef( NodeBase, ExpressionMixin ):
 
                 # TODO: More like "removed_variable and new_constant" probably
                 change_tags = "new_builtin"
-                change_desc = "Module variable '%s' found to be builtin exception reference." % self.variable_name
+                change_desc = """\
+Module variable '%s' found to be builtin exception reference.""" % (
+                    self.variable_name
+                )
             elif self.variable_name in Builtins.builtin_names:
                 from .BuiltinRefNodes import ExpressionBuiltinRef
 
@@ -118,23 +124,30 @@ class ExpressionVariableRef( NodeBase, ExpressionMixin ):
 
                 # TODO: More like "removed_variable and new_constant" probably
                 change_tags = "new_builtin"
-                change_desc = "Module variable '%s' found to be builtin reference." % self.variable_name
+                change_desc = """\
+Module variable '%s' found to be builtin reference.""" % (
+                    self.variable_name
+                )
             elif self.variable_name == "__name__":
                 new_node = ExpressionConstantRef(
-                    constant   = self.variable.getReferenced().getOwner().getFullName(),
+                    constant   = self.variable.getReferenced().getOwner().\
+                        getFullName(),
                     source_ref = self.getSourceReference()
                 )
 
                 change_tags = "new_constant"
-                change_desc = "Replaced read-only module attribute '__name__' with constant value."
+                change_desc = """\
+Replaced read-only module attribute '__name__' with constant value."""
             elif self.variable_name == "__package__":
                 new_node = ExpressionConstantRef(
-                    constant   = self.variable.getReferenced().getOwner().getPackage(),
+                    constant   = self.variable.getReferenced().getOwner().\
+                                   getPackage(),
                     source_ref = self.getSourceReference()
                 )
 
                 change_tags = "new_constant"
-                change_desc = "Replaced read-only module attribute '__package__' with constant value."
+                change_desc = """\
+Replaced read-only module attribute '__package__' with constant value."""
             else:
                 # Probably should give a warning once about it.
                 new_node = self
@@ -145,30 +158,30 @@ class ExpressionVariableRef( NodeBase, ExpressionMixin ):
 
         return self, None, None
 
-    def onContentEscapes( self, constraint_collection ):
+    def onContentEscapes(self, constraint_collection):
         constraint_collection.onVariableContentEscapes( self.variable )
 
-    def isKnownToBeIterable( self, count ):
+    def isKnownToBeIterable(self, count):
         return None
 
-    def mayProvideReference( self ):
+    def mayProvideReference(self):
         # Variables are capable of "asObject0".
         return False
 
-    def mayHaveSideEffects( self ):
+    def mayHaveSideEffects(self):
         # TODO: Remembered traced could tell better.
         return True
 
 
-class ExpressionTargetVariableRef( ExpressionVariableRef ):
+class ExpressionTargetVariableRef(ExpressionVariableRef):
     kind = "EXPRESSION_TARGET_VARIABLE_REF"
 
-    def __init__( self, variable_name, source_ref ):
+    def __init__(self, variable_name, source_ref):
         ExpressionVariableRef.__init__( self, variable_name, source_ref )
 
         self.variable_version = None
 
-    def getDetails( self ):
+    def getDetails(self):
         if self.variable is None:
             return { "name" : self.variable_name }
         else:
@@ -178,7 +191,7 @@ class ExpressionTargetVariableRef( ExpressionVariableRef ):
                 "version"  : self.variable_version
             }
 
-    def makeCloneAt( self, source_ref ):
+    def makeCloneAt(self, source_ref):
         result = self.__class__(
             variable_name = self.variable_name,
             source_ref    = source_ref
@@ -189,178 +202,98 @@ class ExpressionTargetVariableRef( ExpressionVariableRef ):
 
         return result
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
         assert False
 
-    def isTargetVariableRef( self ):
+    def isTargetVariableRef(self):
         return True
 
-    def getVariableVersion( self ):
+    def getVariableVersion(self):
         assert self.variable_version is not None, self
 
         return self.variable_version
 
-    def setVariable( self, variable ):
+    def setVariable(self, variable):
         ExpressionVariableRef.setVariable( self, variable )
 
         self.variable_version = variable.allocateTargetNumber()
         assert self.variable_version is not None
 
 
-class ExpressionTempVariableRef( NodeBase, ExpressionMixin ):
+class ExpressionTempVariableRef(NodeBase, ExpressionMixin):
     kind = "EXPRESSION_TEMP_VARIABLE_REF"
 
-    def __init__( self, variable, source_ref ):
+    def __init__(self, variable, source_ref):
         NodeBase.__init__( self, source_ref = source_ref )
 
         self.variable = variable
 
-    def getDetails( self ):
+    def getDetails(self):
         return { "name" : self.variable.getName() }
 
-    def getDetail( self ):
+    def getDetail(self):
         return self.variable.getName()
 
-    def makeCloneAt( self, source_ref ):
+    def makeCloneAt(self, source_ref):
         return self.__class__(
             variable   = self.variable,
             source_ref = source_ref
         )
 
-    def getVariableName( self ):
+    def getVariableName(self):
         return self.variable.getName()
 
-    def getVariable( self ):
+    def getVariable(self):
         return self.variable
 
-    def isTargetVariableRef( self ):
+    def isTargetVariableRef(self):
         return False
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
+        constraint_collection.onVariableUsage( self )
+
         # Nothing to do here.
         return self, None, None
 
-    def onContentEscapes( self, constraint_collection ):
+    def onContentEscapes(self, constraint_collection):
         constraint_collection.onVariableContentEscapes( self.variable )
 
-    def mayRaiseException( self, exception_type ):
+    def mayRaiseException(self, exception_type):
         # Can't happen
         return False
 
-    def isKnownToBeIterableAtMin( self, count ):
+    def isKnownToBeIterableAtMin(self, count):
+        # TODO: See through the variable current trace.
         return None
 
-        friend = constraint_collection.getVariableValueFriend( self.variable )
-
-        if friend is not None:
-            return friend.isKnownToBeIterableAtMin(
-                count                 = count,
-                constraint_collection = constraint_collection
-            )
-        else:
-            return None
-
-    def isKnownToBeIterableAtMax( self, count ):
+    def isKnownToBeIterableAtMax(self, count):
+        # TODO: See through the variable current trace.
         return None
 
-        friend = constraint_collection.getVariableValueFriend( self.variable )
-
-        if friend is not None:
-            return friend.isKnownToBeIterableAtMax(
-                count                 = count,
-                constraint_collection = constraint_collection
-            )
-        else:
-            return None
-
-    def getIterationNext( self, constraint_collection ):
-        return None
-
-        friend = constraint_collection.getVariableValueFriend( self.variable )
-
-        if friend is not None:
-            return friend.getIterationNext(
-                constraint_collection = constraint_collection
-            )
-        else:
-            return None
-
-
     # Python3 only, it updates temporary variables that are closure variables.
-    def setVariable( self, variable ):
+    def setVariable(self, variable):
         self.variable = variable
 
 
-class ExpressionTargetTempVariableRef( ExpressionTempVariableRef ):
+class ExpressionTargetTempVariableRef(ExpressionTempVariableRef):
     kind = "EXPRESSION_TARGET_TEMP_VARIABLE_REF"
 
-    def __init__( self, variable, source_ref ):
+    def __init__(self, variable, source_ref):
         ExpressionTempVariableRef.__init__( self, variable, source_ref )
 
         self.variable_version = variable.allocateTargetNumber()
 
-    def computeExpression( self, constraint_collection ):
+    def computeExpression(self, constraint_collection):
         assert False, self.parent
 
-    def isTargetVariableRef( self ):
+    def isTargetVariableRef(self):
         return True
 
-    def getVariableVersion( self ):
+    def getVariableVersion(self):
         return self.variable_version
 
     # Python3 only, it updates temporary variables that are closure variables.
-    def setVariable( self, variable ):
+    def setVariable(self, variable):
         ExpressionTempVariableRef.setVariable( self, variable )
 
         self.variable_version = self.variable.allocateTargetNumber()
-
-
-class StatementTempBlock( StatementChildrenHavingBase ):
-    kind = "STATEMENT_TEMP_BLOCK"
-
-    named_children = ( "body", )
-
-    def __init__( self, source_ref ):
-        StatementChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "body" : None
-            },
-            source_ref = source_ref.atInternal()
-        )
-
-        self.temp_variables = {}
-
-    getBody = StatementChildrenHavingBase.childGetter( "body" )
-    setBody = StatementChildrenHavingBase.childSetter( "body" )
-
-    def getTempVariable( self, name ):
-        assert name not in self.temp_variables, name
-
-        result = Variables.TempVariable(
-            owner         = self,
-            variable_name = name
-        )
-
-        self.temp_variables[ name ] = result
-
-        return result
-
-    def getTempVariables( self ):
-        return self.temp_variables.values()
-
-    def mayHaveSideEffects( self ):
-        return self.getBody().mayHaveSideEffects()
-
-    def computeStatement( self, constraint_collection ):
-        from nuitka.optimizations.ConstraintCollections import ConstraintCollectionTempBlock
-
-        collection_temp_block = ConstraintCollectionTempBlock(
-            constraint_collection
-        )
-        collection_temp_block.process( self )
-
-        if self.getBody() is None:
-            return None, "new_statements", "Removed empty temporary block"
-
-        return self, None, None
