@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,7 +15,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-def func( arg1, arg2, arg3, **star ):
+def func(arg1, arg2, arg3, **star):
    """ Some documentation. """
 
    pass
@@ -41,7 +41,10 @@ print "Reading its dict", func.my_value
 print "func_code", func.func_code, func.func_code.co_argcount
 print dir( func.func_code )
 
-def func2( arg1, arg2 = "default_arg2", arg3 = "default_arg3" ):
-   pass
+def func2(arg1, arg2 = "default_arg2", arg3 = "default_arg3"):
+   x = 1
+   return x
 
 print "func_defaults", func2.__defaults__, func2.func_defaults
+
+print "function varnames", func2.__code__.co_varnames
