@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -18,22 +18,22 @@
 
 print "Generator expression that demonstrates the timing:"
 def iteratorCreationTiming():
-    def getIterable( x ):
+    def getIterable(x):
         print "Getting iterable", x
         return Iterable( x )
 
     class Iterable:
-        def __init__( self, x ):
+        def __init__(self, x):
             self.x = x
             self.values = range( x )
             self.count = 0
 
-        def __iter__( self ):
+        def __iter__(self):
             print "Giving iter now", self.x
 
             return self
 
-        def next( self ):
+        def next(self):
             print "Next of", self.x, "is", self.count
 
             if len( self.values ) > self.count:
@@ -45,7 +45,7 @@ def iteratorCreationTiming():
 
                 raise StopIteration
 
-        def __del__( self ):
+        def __del__(self):
             print "Deleting", self.x
 
 
@@ -63,6 +63,14 @@ def iteratorCreationTiming():
     except StopIteration:
         print "Use past end gave StopIteration as expected"
 
+        try:
+            import inspect
+            print "Generator state then is", inspect.getgeneratorstate( gen )
+        except AttributeError:
+            pass
+
+        print "Its frame is now", gen.gi_frame
+
     print "Early aborting generator"
 
     gen2 = ( (y,z) for y in getIterable( 3 ) for z in getIterable( 2 ) )
@@ -100,11 +108,20 @@ def genexprSend():
     print "Send return value", y
     print "And then next gave", x.next()
 
-    print "Throwing an exception from it."
+    print "Throwing an exception to it."
+    try:
+        x.throw( 2, 2, None )
+        assert False
+    except TypeError, e:
+        print "Gave expected TypeError:", e
+
+    print "Throwing an exception to it."
     try:
         x.throw( ValueError, 2, None )
     except ValueError, e:
-        print "Gave expected ValueError with text:", e
+        print "Gave expected ValueError:", e
+
+
 
     try:
         x.next()
@@ -139,7 +156,7 @@ genexprClose()
 
 def genexprThrown():
 
-    def checked( z ):
+    def checked(z):
         if z == 3:
             raise ValueError
 
