@@ -26,12 +26,15 @@ assert inspect.isfunction( compiledFunction ) is True
 assert isinstance( compiledFunction, types.FunctionType )
 assert isinstance( compiledFunction, ( int, types.FunctionType ) )
 
+# Even this works.
+assert type( compiledFunction ) == types.FunctionType
+
 class compiledClass:
-   def compiledMethod( self ):
+   def compiledMethod(self):
       pass
 
 assert inspect.isfunction( compiledClass ) is False
-assert isinstance( compiledClass,types.FunctionType ) is False
+assert isinstance( compiledClass, types.FunctionType ) is False
 
 assert inspect.ismethod( compiledFunction ) is False
 assert inspect.ismethod( compiledClass ) is False
@@ -39,6 +42,8 @@ assert inspect.ismethod( compiledClass ) is False
 assert inspect.ismethod( compiledClass.compiledMethod ) == ( sys.version_info < ( 3, ) )
 assert inspect.ismethod( compiledClass().compiledMethod ) is True
 
+assert bool( type( compiledClass.compiledMethod ) == types.MethodType ) == ( sys.version_info < ( 3, ) )
+
 def compiledGenerator():
    yield 1
 
@@ -46,6 +51,7 @@ assert inspect.isfunction( compiledGenerator ) is True
 assert inspect.isgeneratorfunction( compiledGenerator ) is True
 
 assert isinstance( compiledGenerator(), types.GeneratorType ) is True
+assert type( compiledGenerator() ) == types.GeneratorType
 assert isinstance( compiledGenerator, types.GeneratorType ) is False
 
 assert inspect.ismethod( compiledGenerator() ) is False
@@ -64,7 +70,7 @@ someFunction()
 import sys
 
 class C:
-    print "Class locals", str( sys._getframe().f_locals ).replace( "'__locals__': {...}, ", "" ).replace( "'__qualname__': 'C', ", "" )
+    print "Class locals", str( sys._getframe().f_locals ).replace( ", '__locals__': {...}", "" ).replace( "'__qualname__': 'C', ", "" )
     print "Class flags", sys._getframe().f_code.co_flags | 64
 
 def f():
@@ -73,7 +79,7 @@ def f():
 
 f()
 
-def displayDict( d ):
+def displayDict(d):
     d = dict(d)
     if "__loader__" in d:
         d[ "__loader__" ] = "<loader removed>"
