@@ -25,10 +25,11 @@ print "doc:", __doc__
 print "filename:", __file__
 print "builtins:", __builtins__
 print "debug", __debug__
+print "debug in builtins", __builtins__.__debug__
 
-print "__initialised__",
+print "__initializing__",
 try:
-    print __initialised__
+    print __initializing__
 except NameError:
     print "not found"
 
@@ -36,7 +37,7 @@ def checkFromFunction():
     frame = sys._getframe(1)
     locals = frame.f_locals
 
-    def displayDict( d ):
+    def displayDict(d):
         d = dict(d)
         if "__loader__" in d:
             d[ "__loader__" ] = "<loader removed>"
