@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -33,7 +33,7 @@ try:
 except TypeError, e:
     print repr(e)
 
-def functionOneParameter( a ):
+def functionOneParameter(a):
     print a
 
 print "Call a function with one parameter with two plain arguments:"
@@ -43,9 +43,15 @@ try:
 except TypeError, e:
     print repr(e)
 
+print "Call a function with one parameter too many, and duplicate arguments:"
+try:
+    functionOneParameter(6, a=4, *(1, 2, 3))
+except TypeError, e:
+    print repr(e)
+
 print "Call a function with two parameters with three plain arguments:"
 
-def functionTwoParameters( a, b ):
+def functionTwoParameters(a, b):
     print a, b
 
 try:
@@ -60,6 +66,14 @@ try:
 except TypeError, e:
     print repr(e)
 
+print "Call a function with two parameters with three plain arguments:"
+
+try:
+    functionTwoParameters( 1, 2, 3 )
+except TypeError, e:
+    print repr(e)
+
+
 print "Call a function with two parameters with one keyword argument:"
 
 try:
@@ -75,13 +89,13 @@ except TypeError, e:
     print repr(e)
 
 class MethodContainer:
-    def methodNoParameters( self ):
+    def methodNoParameters(self):
         pass
 
-    def methodOneParameter( self, a ):
+    def methodOneParameter(self, a):
         print a
 
-    def methodTwoParameters( self, a, b ):
+    def methodTwoParameters(self, a, b):
         print a, b
 
 obj = MethodContainer()
@@ -135,7 +149,7 @@ try:
 except TypeError, e:
     print repr(e)
 
-def functionPosBothStarArgs( a, b, c, *l, **d ):
+def functionPosBothStarArgs(a, b, c, *l, **d):
     print a, b, c, l, d
 
 l = [2]
@@ -148,10 +162,116 @@ try:
 except TypeError, e:
     print repr(e)
 
-def functionWithDefaults( a, b, c, d = 3 ):
+print "Call a function with defaults with too little arguments:"
+
+def functionWithDefaults(a, b, c, d = 3):
     print a, b, c, d
 
 try:
     functionWithDefaults( 1 )
 except TypeError, e:
     print repr(e)
+
+print "Call a function with defaults with too many arguments:"
+
+try:
+    functionWithDefaults( 1 )
+except TypeError, e:
+    print repr(e)
+
+print "Complex call with invalid star list and star arguments:"
+
+try:
+    a = 1
+    b = 2.0
+
+    functionWithDefaults(1,c=3,*a,**b)
+except TypeError, e:
+    print repr(e)
+
+try:
+    a = 1
+    b = 2.0
+
+    functionWithDefaults(1,*a,**b)
+except TypeError, e:
+    print repr(e)
+
+try:
+    a = 1
+    b = 2.0
+
+    functionWithDefaults(c=1, *a,**b)
+except TypeError, e:
+    print repr(e)
+
+try:
+    a = 1
+    b = 2.0
+
+    functionWithDefaults(*a,**b)
+except TypeError, e:
+    print repr(e)
+
+try:
+    a = 1
+
+    functionWithDefaults(*a)
+except TypeError, e:
+    print repr(e)
+
+
+try:
+    a = 1
+
+    MethodContainer(*a)
+except TypeError, e:
+    print repr(e)
+
+
+try:
+    a = 1
+
+    MethodContainer()(*a)
+except TypeError, e:
+    print repr(e)
+
+try:
+    a = 1
+
+    MethodContainer.methodTwoParameters(*a)
+except TypeError, e:
+    print repr(e)
+
+try:
+    a = 1
+
+    None(*a)
+except TypeError, e:
+    print repr(e)
+
+
+try:
+    a = 1
+
+    None(**a)
+except TypeError, e:
+    print repr(e)
+
+print "Call object with name as both keyword and in star dict argument:"
+try:
+    a = {"a" : 3}
+
+    None(a=2, **a)
+except TypeError, e:
+    print repr(e)
+
+print "Call function with only defaulted value given as keyword argument:"
+
+def functionwithTwoArgsOneDefaulted(a, b=5):
+    pass
+
+try:
+    functionwithTwoArgsOneDefaulted(b=12)
+except TypeError, e:
+    print repr(e)
