@@ -16,7 +16,7 @@
 #     limitations under the License.
 #
 
-def kwfunc( a, *, k ):
+def kwfunc(a, *, k):
     pass
 
 print( "Call function with mixed arguments with too wrong keyword argument." )
@@ -48,12 +48,27 @@ try:
 except TypeError as e:
     print( repr(e) )
 
-def kwfuncdefaulted( a, b = None, *, c = None ):
+def kwfuncdefaulted(a, b = None, *, c = None):
     pass
 
 print( "Call function with mixed arguments and defaults but too many position arguments." )
 
 try:
-    kwfuncdefaulted( 1, 2, 3 )
+    kwfuncdefaulted(1, 2, 3)
+except TypeError as e:
+    print( repr(e) )
+
+def kwfunc2(a, *, k, l, m):
+    pass
+
+print( "Call function with mixed arguments with too little positional and keyword-only arguments." )
+
+try:
+    kwfunc2( 1, l = 2 )
+except TypeError as e:
+    print( repr(e) )
+
+try:
+    kwfunc2( 1 )
 except TypeError as e:
     print( repr(e) )
