@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -42,3 +42,8 @@ except Exception, e:
     print "then occured", repr(e)
 
 print "No newline at the end",
+
+x = 1
+print """
+New line is no soft space, is it
+""", x
