@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -17,6 +17,10 @@
 #
 import sys, gc
 
+if not hasattr( sys, "gettotalrefcount" ):
+    print( "Warning, using non-debug Python makes this test ineffective." )
+    sys.gettotalrefcount = lambda : 0
+
 gc.disable()
 
 def simpleFunction1():
@@ -43,7 +47,7 @@ def simpleFunction4():
 
     return contained
 
-def simpleFunction5( a = 1*2 ):
+def simpleFunction5(a = 1*2):
     c = 1
     f = [ a, a + c ]
 
@@ -55,7 +59,7 @@ def simpleFunction6():
         pass
 
 
-def simpleFunction7( b = 1 ):
+def simpleFunction7(b = 1):
     for b in range(6):
         pass
 
@@ -63,13 +67,13 @@ def simpleFunction8():
     c = []
     c.append( x )
 
-def simpleFunction9( a = 1*2 ):
+def simpleFunction9(a = 1*2):
     if a == a:
         pass
 
 u = None
 
-def simpleFunction10( a = 1*2 ):
+def simpleFunction10(a = 1*2):
     x = [u for u in range(8)]
 
 def simpleFunction11():
@@ -86,7 +90,7 @@ def simpleFunction12():
 def cond():
     return 1
 
-def simpleFunction13( a = 1*2 ):
+def simpleFunction13(a = 1*2):
     pass
 
 def simpleFunction14p(x):
@@ -134,7 +138,7 @@ def simpleFunction18():
     closured = 1
 
     class NonEmptyClass:
-        def __init__( self, a, b ):
+        def __init__(self, a, b):
             self.a = a
             self.b = b
 
@@ -158,10 +162,10 @@ def simpleFunction20():
 
 def simpleFunction21():
     class EmptyBaseClass:
-        def base( self ):
+        def base(self):
             return 3
 
-    class EmptyObjectClass( EmptyBaseClass ):
+    class EmptyObjectClass(EmptyBaseClass):
         pass
 
     result = EmptyObjectClass()
@@ -176,7 +180,7 @@ def simpleFunction22():
 def simpleFunction23():
     not 2
 
-def simpleFunction24p( x ):
+def simpleFunction24p(x):
     pass
 
 def simpleFunction24():
@@ -187,7 +191,7 @@ def simpleFunction25():
     class X:
         f = 1
 
-    def inplace_adder( b ):
+    def inplace_adder(b):
         X.f += b
 
     return inplace_adder( 6**8 )
@@ -197,7 +201,7 @@ def simpleFunction26():
     class X:
         f = [ 5 ]
 
-    def inplace_adder( b ):
+    def inplace_adder(b):
         X.f += b
 
     return inplace_adder( [ 1, 2 ] )
@@ -205,7 +209,7 @@ def simpleFunction26():
 def simpleFunction27():
     a = { "g": 8 }
 
-    def inplace_adder( b ):
+    def inplace_adder(b):
         a[ "g" ] += b
 
     return inplace_adder( 3 )
@@ -213,7 +217,7 @@ def simpleFunction27():
 def simpleFunction28():
     a = { "g": [ 8 ], "h": 2 }
 
-    def inplace_adder( b ):
+    def inplace_adder(b):
         a[ "g" ] += b
 
     return inplace_adder( [ 3, 5 ] )
@@ -294,11 +298,11 @@ def simpleFunction38():
    class Base:
       pass
 
-   class Parent( Base ):
+   class Parent(Base):
       pass
 
 def simpleFunction39():
-   class Parent( object ):
+   class Parent(object):
       pass
 
 
@@ -325,13 +329,13 @@ def simpleFunction43():
    a.b = 1
 
 def simpleFunction44():
-   def nested_args_function( (a,b), c ):
+   def nested_args_function((a,b), c):
       return a, b, c
 
    nested_args_function( ( 1, 2 ), 3 )
 
 def simpleFunction45():
-   def nested_args_function( (a,b), c ):
+   def nested_args_function((a,b), c):
       return a, b, c
 
    try:
@@ -340,7 +344,7 @@ def simpleFunction45():
       pass
 
 def simpleFunction46():
-   def nested_args_function( (a,b), c ):
+   def nested_args_function((a,b), c):
       return a, b, c
 
    try:
@@ -368,7 +372,7 @@ def simpleFunction47():
 
 def simpleFunction48():
    class BlockExceptions:
-      def __enter__( self ):
+      def __enter__(self):
          pass
       def __exit__( self, exc, val, tb):
          return True
@@ -461,9 +465,110 @@ def simpleFunction57():
    f(2)
    f(3,4)
 
+def simpleFunction58():
+   a = 3
+   b = 5
+
+   try:
+       a = a * 2
+
+       return a
+   finally:
+      a / b
+
+
+def simpleFunction59():
+   a = 3
+   b = 5
+
+   try:
+       a = a * 2
+
+       return a
+   finally:
+       return a / b
+
+
+def simpleFunction60():
+   try:
+      raise ValueError(1,2,3), ValueError(1,2,3)
+   except Exception:
+      pass
+
+def simpleFunction61():
+   try:
+      raise ValueError, 2, None
+   except Exception:
+      pass
+
+def simpleFunction62():
+   try:
+      raise ValueError, 2, 3
+   except Exception:
+      pass
+
+class X:
+   def __del__(self):
+      # Super used to reference leak.
+      x = super()
+
+      raise ValueError, ValueError(1)
+
+def simpleFunction63():
+   def superUser():
+      X()
+
+   try:
+      superUser()
+   except Exception:
+      pass
+
+def simpleFunction64():
+    x = 2
+    y = 3
+    z = eval( "x * y" )
+
+def simpleFunction65():
+    import array
+
+    a = array.array("b", "")
+    assert a == eval(repr(a), {"array": array.array})
+
+    d = {
+        "x" : 2,
+        "y" : 3
+    }
+    z = eval( repr(d), d )
+
+
+def simpleFunction66():
+    import types
+    return type(simpleFunction65) == types.FunctionType
+
 x = 17
 
-def checkReferenceCount( checked_function, max_rounds = 10 ):
+m1 = {}
+m2 = {}
+
+def snapObjRefCntMap(before):
+   if before:
+      global m1
+      m = m1
+   else:
+      global m2
+      m = m2
+
+   for x in gc.get_objects():
+      if x is m1:
+         continue
+
+      if x is m2:
+         continue
+
+      m[ str( x ) ] = sys.getrefcount( x )
+
+
+def checkReferenceCount(checked_function, max_rounds = 10):
    assert sys.exc_info() == ( None, None, None ), sys.exc_info()
 
    print checked_function.func_name + ":",
@@ -471,13 +576,17 @@ def checkReferenceCount( checked_function, max_rounds = 10 ):
    ref_count1 = 17
    ref_count2 = 17
 
+   explain = False
+
    for count in range( max_rounds ):
       x1 = 0
       x2 = 0
 
       gc.collect()
       ref_count1 = sys.gettotalrefcount()
-      x1 = len( gc.get_objects() )
+
+      if explain and count == max_rounds - 1:
+         snapObjRefCntMap( True )
 
       checked_function()
 
@@ -485,20 +594,33 @@ def checkReferenceCount( checked_function, max_rounds = 10 ):
 
       gc.collect()
 
+      if explain and count == max_rounds - 1:
+         snapObjRefCntMap( False )
+
       ref_count2 = sys.gettotalrefcount()
 
       if ref_count1 == ref_count2:
          print "PASSED"
          break
 
-      x2 = len( gc.get_objects() )
-
-      if False and count == max_rounds - 1:
-         print gc.get_objects()[ x1 : x2 ][0].f_code
-
       # print count, ref_count1, ref_count2
    else:
-      print "FAILED", ref_count1, ref_count2
+      print "FAILED", ref_count1, ref_count2, "leaked", ref_count2 - ref_count1
+
+      if explain:
+         assert m1
+         assert m2
+
+         for key in m1.keys():
+            if key not in m2:
+               print "*" * 80
+               print key
+            elif m1[key] != m2[key]:
+               print "*" * 80
+               print key
+            else:
+               pass
+               # print m1[key]
 
    assert sys.exc_info() == ( None, None, None ), sys.exc_info()
 
@@ -564,3 +686,25 @@ checkReferenceCount( simpleFunction55 )
 # we accept for now.
 # checkReferenceCount( simpleFunction56 )
 checkReferenceCount( simpleFunction57 )
+checkReferenceCount( simpleFunction58 )
+checkReferenceCount( simpleFunction59 )
+checkReferenceCount( simpleFunction60 )
+checkReferenceCount( simpleFunction61 )
+checkReferenceCount( simpleFunction62 )
+
+# Avoid unraisable output.
+old_stderr = sys.stderr
+try:
+   sys.stderr = open( "/dev/null", "wb" )
+except Exception: # Windows
+    checkReferenceCount( simpleFunction63 )
+else:
+    checkReferenceCount( simpleFunction63 )
+
+    new_stderr = sys.stderr
+    sys.stderr = old_stderr
+    new_stderr.close()
+
+checkReferenceCount( simpleFunction64 )
+checkReferenceCount( simpleFunction65 )
+checkReferenceCount( simpleFunction66 )
