@@ -17,13 +17,13 @@
 #     limitations under the License.
 #
 
-def plain_list_dict_args_function( plain, *arg_list, **arg_dict ):
-    print plain, arg_list, arg_dict
+def plain_list_dict_args_function(plain, *arg_list, **arg_dict):
+    print "plain", plain, "arg_list", arg_list, "arg_dict", arg_dict
 
-def plain_list_args_function( plain, *arg_list ):
+def plain_list_args_function(plain, *arg_list):
     print plain, arg_list
 
-def plain_dict_args_function( plain, **arg_dict ):
+def plain_dict_args_function(plain, **arg_dict):
     print plain, arg_dict
 
 
@@ -40,13 +40,13 @@ plain_list_dict_args_function( 1, 2, z = 3 )
 plain_list_dict_args_function( 1, 2, 3 )
 plain_list_dict_args_function( 1, a = 2, b = 3, c = 4 )
 
-def list_dict_args_function( *arg_list, **arg_dict ):
+def list_dict_args_function(*arg_list, **arg_dict):
     print arg_list, arg_dict
 
-def list_args_function( *arg_list ):
+def list_args_function(*arg_list):
     print arg_list
 
-def dict_args_function( **arg_dict ):
+def dict_args_function(**arg_dict):
     print arg_dict
 
 print "Function with plain arg and varargs dict:"
