@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -18,24 +18,27 @@
 
 x = 0
 
+# This is used to trace the exact interaction with the context manager to uncover and
+# decide orddering and correctness of calls.
 class MyContextManager(object):
-    def __getattribute__( self, attribute_name ):
+    def __getattribute__(self, attribute_name):
         print "Asking context manager attribute", attribute_name
         return object.__getattribute__( self, attribute_name )
 
-    def __enter__( self ):
+    def __enter__(self):
         global x
         x += 1
 
-        print "Entered context manager", x
+        print "Entered context manager with counter value", x
 
         return x
 
-    def __exit__( self, exc_type, exc_value, traceback ):
+    def __exit__(self, exc_type, exc_value, traceback):
         print "Exit sees", exc_type, exc_value, traceback
 
         return False
 
+print "Use context manager and raise exception in the body:"
 with MyContextManager() as x:
     print "x has become", x
 
@@ -50,6 +53,7 @@ except Exception, e:
 
 l = range(3)
 
+print "Use context manager and assign to subscription target:"
 with MyContextManager() as l[0]:
     print "Complex assignment target works", l[0]
 
@@ -61,6 +65,7 @@ except BaseException as e:
     print "Caught base exception", repr(e)
 
 
+print "Use context manager and fail to assign to attribute:"
 try:
     import sys
     with MyContextManager() as l.wontwork:
@@ -68,15 +73,26 @@ try:
 except BaseException as e:
     print "Caught base exception", repr(e)
 
+print "Use context manager to do nothing inside:"
 with MyContextManager() as x:
     pass
 
+# Use context manager and fail to assign.
+def returnFromContextBlock():
+    # Use context manager to do nothing.
+    with MyContextManager() as x:
+        return 7
+
+print "Use context manager to return value:"
+r = returnFromContextBlock()
+print "Return value", r
+
 class NonContextManager1:
-    def __enter__( self ):
+    def __enter__(self):
         return self
 
 class NonContextManager2:
-    def __exit__( self ):
+    def __exit__(self):
         return self
 
 try:
@@ -101,10 +117,10 @@ except Exception, e:
     print repr(e)
 
 class MeanContextManager:
-    def __enter__( self ):
+    def __enter__(self):
         raise ValueError( "Nah, I won't play" )
 
-    def __exit__( self ):
+    def __exit__(self):
         print "Called exit, yes"
 
 try:
