@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -16,7 +16,12 @@
 #     limitations under the License.
 #
 
-__import__( "dash-module" )
+__import__("dash-module")
 
 b = "dash-module"
-__import__( b )
+__import__(b)
+
+__import__("plus+module")
+
+c = "plus+module"
+__import__(c)
