@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -16,9 +16,12 @@
 #     limitations under the License.
 #
 
-print( "*** Child2: Begin" )
-
-import Child1
+print( "*** Child2: Begin", __name__ )
+try:
+    import Child1
+except ImportError:
+    print( "This must be Python3, doing local import then." )
+    from . import Child1
 
 print( "*** Child2: Child2 is in", __package__ )
 print( "*** Child2: Imported nearby child", Child1 )
