@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -16,4 +16,8 @@
 #     limitations under the License.
 #
 
-import PackageLocal
+try:
+    import PackageLocal
+except ImportError:
+    print( "This must be Python3, doing local import then." )
+    from . import PackageLocal
