@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,3 +15,8 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
+
+print( "This must be Python3.3, which no longer needs __init__.py to accept a package." )
+
+import sys
+print( "The parent path is", sys.modules[ "some_package"].__path__ )
