@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -17,7 +17,7 @@
 #
 
 from foo import bar, not_overloaded
-print bar, not_overloaded
+print( bar, not_overloaded )
 
 from foo import bar2, not_overloaded
-print bar2
+print( bar2 )
