@@ -1,4 +1,4 @@
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,13 +15,13 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-print "some_package.star_importing, doing the star import"
+print( "some_package.star_importing, doing the star import" )
 
-print "Before", sorted( dir() )
+print( "Before", sorted( dir() ) )
 
-from posix import *
+from .pyexpat import *
 
 lala = 1
-print "After", sorted( dir() )
+print( "After", sorted( dir() ) )
 
-print "Finished"
+print( "Finished" )
