@@ -1,5 +1,5 @@
 #!/usr/bin/env python
-#     Copyright 2012, Kay Hayen, mailto:kayhayen@gmx.de
+#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -17,59 +17,47 @@
 #     limitations under the License.
 #
 
-from __future__ import print_function
-
 import os, sys, shutil, tempfile, time, difflib, subprocess
 
-# Go its own directory, to have it easy with path knowledge.
-os.chdir( os.path.dirname( os.path.abspath( __file__ ) ) )
+# Find common code relative in file system. Not using packages for test stuff.
+sys.path.insert(
+    0,
+    os.path.normpath(
+        os.path.join(
+            os.path.dirname( os.path.abspath( __file__ ) ),
+            ".."
+        )
+    )
+)
+from test_common import (
+    my_print,
+    setup,
+    getTempDir
+)
+
+python_version = setup()
+
+# TODO: This ought to no longer be necessary, removing it could highlight bugs.
+# No random hashing, it makes comparing outputs futile.
+if "PYTHONHASHSEED" not in os.environ:
+    os.environ[ "PYTHONHASHSEED" ] = "0"
 
 nuitka_main_path = os.path.join( "..", "..", "bin", "nuitka" )
 
-if "PYTHON" not in os.environ:
-    os.environ[ "PYTHON" ] = "python"
+tmp_dir = getTempDir()
 
-def check_output(*popenargs, **kwargs):
-    from subprocess import Popen, PIPE, CalledProcessError
-
-    if 'stdout' in kwargs:
-        raise ValueError('stdout argument not allowed, it will be overridden.')
-    process = Popen(stdout=PIPE, *popenargs, **kwargs)
-    output, unused_err = process.communicate()
-    retcode = process.poll()
-    if retcode:
-        cmd = kwargs.get("args")
-        if cmd is None:
-            cmd = popenargs[0]
-        raise CalledProcessError(retcode, cmd, output=output)
-    return output
-
-version_output = check_output(
-    [ os.environ[ "PYTHON" ], "--version" ],
-    stderr = subprocess.STDOUT
-)
-
-python_version = version_output.split()[1]
-
-print( "Using concrete python", python_version )
-
-tmp_dir = tempfile.gettempdir()
-
-# Try to avoid RAM disk /tmp and use the disk one instead.
-if tmp_dir == "/tmp" and os.path.exists( "/var/tmp" ):
-    tmp_dir = "/var/tmp"
-
-# Could detect this more automatic.
+# TODO: Could detect this more automatic.
 PACKAGE_LIST = (
     'nuitka',
     'nuitka/nodes',
+    'nuitka/tree',
     'nuitka/build',
+    'nuitka/freezer',
     'nuitka/gui',
     'nuitka/codegen',
     'nuitka/codegen/templates',
-    'nuitka/transform',
-    'nuitka/transform/optimizations',
-    'nuitka/transform/finalizations',
+    'nuitka/optimizations',
+    'nuitka/finalizations',
 )
 
 def diffRecursive( dir1, dir2 ):
@@ -82,7 +70,9 @@ def diffRecursive( dir1, dir2 ):
         done.add( path1 )
 
         # Skip these binary files of course.
-        if filename.endswith( ".o" ) or filename.endswith( ".os" ):
+        if filename.endswith( ".o" ) or \
+           filename.endswith( ".os" ) or \
+           filename.endswith( ".obj" ):
             continue
 
         # Skip scons build database
@@ -112,7 +102,7 @@ def diffRecursive( dir1, dir2 ):
 
             if result:
                 for line in result:
-                    print( line )
+                    my_print( line )
 
                 sys.exit( 1 )
         else:
@@ -129,7 +119,7 @@ def diffRecursive( dir1, dir2 ):
             sys.exit( "Only in %s: %s" % ( dir2, filename ))
 
 def executePASS1():
-    print( "PASS 1: Compiling from compiler running from .py files." )
+    my_print( "PASS 1: Compiling from compiler running from .py files." )
 
     base_dir = os.path.join( "..", ".." )
 
@@ -151,23 +141,29 @@ def executePASS1():
                 os.unlink( path )
 
         for filename in sorted( os.listdir( source_dir ) ):
-            if not filename.endswith( ".py" ):
+            if not filename.endswith(".py"):
+                continue
+
+            if filename.startswith(".#"):
                 continue
 
             path = os.path.join( source_dir, filename )
 
             if filename != "__init__.py":
-                print( "Compiling", path )
+                my_print( "Compiling", path )
+
+                command = [
+                    os.environ[ "PYTHON" ],
+                    nuitka_main_path,
+                    "--module",
+                    "--recurse-none",
+                    "--output-dir=%s" % target_dir,
+                    path
+                ]
+                command += os.environ.get( "NUITKA_EXTRA_OPTIONS", "" ).split()
 
                 result = subprocess.call(
-                    "%s %s %s --output-dir %s %s" % (
-                        os.environ[ "PYTHON" ],
-                        nuitka_main_path,
-                        path,
-                        target_dir,
-                        os.environ.get( "NUITKA_EXTRA_OPTIONS", "" )
-                    ),
-                    shell = True
+                    command
                 )
 
                 if result != 0:
@@ -176,17 +172,21 @@ def executePASS1():
                 shutil.copyfile( path, os.path.join( target_dir, filename ) )
 
 
-    print( "Compiling", nuitka_main_path )
+    my_print( "Compiling", nuitka_main_path )
+
+    shutil.copyfile( nuitka_main_path, "nuitka.py" )
+
+    command = [
+        os.environ[ "PYTHON" ],
+        nuitka_main_path,
+        "--recurse-none",
+        "--output-dir=.",
+        "nuitka.py"
+    ]
+    command += os.environ.get( "NUITKA_EXTRA_OPTIONS", "" ).split()
 
     result = subprocess.call(
-        "%s %s %s --exe --recurse-none --output-dir %s %s" % (
-            os.environ[ "PYTHON" ],
-            nuitka_main_path,
-            nuitka_main_path,
-            ".",
-            os.environ.get( "NUITKA_EXTRA_OPTIONS", "" )
-        ),
-        shell = True
+        command
     )
 
     if result != 0:
@@ -222,13 +222,16 @@ def compileAndCompareWith( nuitka ):
         source_dir = os.path.join( base_dir, package )
 
         for filename in sorted( os.listdir( source_dir ) ):
-            if not filename.endswith( ".py" ):
+            if not filename.endswith(".py"):
+                continue
+
+            if filename.startswith(".#"):
                 continue
 
             path = os.path.join( source_dir, filename )
 
             if filename != "__init__.py":
-                print( "Compiling", path )
+                my_print( "Compiling", path )
 
                 target = filename.replace( ".py", ".build" )
 
@@ -237,14 +240,17 @@ def compileAndCompareWith( nuitka ):
                 if os.path.exists( target_dir ):
                     shutil.rmtree( target_dir )
 
+                command = [
+                    nuitka,
+                    "--module",
+                    "--recurse-none",
+                    "--output-dir=%s"% tmp_dir,
+                    path
+                ]
+                command += os.environ.get( "NUITKA_EXTRA_OPTIONS", "" ).split()
+
                 result = subprocess.call(
-                    "%s %s --output-dir %s %s" % (
-                        nuitka,
-                        path,
-                        tmp_dir,
-                        os.environ.get( "NUITKA_EXTRA_OPTIONS", "" )
-                    ),
-                    shell = True
+                    command
                 )
 
                 if result != 0:
@@ -252,17 +258,34 @@ def compileAndCompareWith( nuitka ):
 
                 diffRecursive( os.path.join( package, target ), target_dir )
 
-                shutil.rmtree( target_dir )
+                shutil.rmtree(target_dir)
+
+                if os.name == "nt":
+                    target_filename = filename.replace(".py", ".pyd")
+                else:
+                    target_filename = filename.replace(".py", ".so")
+
+                os.unlink(os.path.join(tmp_dir, target_filename))
+
 
 def executePASS2():
-    print( "PASS 2: Compiling from compiler running from .exe and many .so files." )
+    my_print( "PASS 2: Compiling from compiler running from .exe and many .so files." )
+
+    # Windows will load the compiled modules (pyd) only from PYTHONPATH, so we
+    # have to add it.
+    if os.name == "nt":
+        os.environ[ "PYTHONPATH" ] = ":".join( PACKAGE_LIST )
 
     compileAndCompareWith( os.path.join( ".", "nuitka.exe" ) )
 
-    print( "OK." )
+    # Undo the damage from above.
+    if os.name == "nt":
+        del os.environ[ "PYTHONPATH" ]
+
+    my_print( "OK." )
 
 def executePASS3():
-    print( "PASS 3: Compiling from compiler running from .py files to single .exe." )
+    my_print( "PASS 3: Compiling from compiler running from .py files to single .exe." )
 
     exe_path = os.path.join( tmp_dir, "nuitka.exe" )
 
@@ -276,16 +299,18 @@ def executePASS3():
 
     path = os.path.join( "..", "..", "bin", "nuitka" )
 
-    print( "Compiling", path )
+    my_print( "Compiling", path )
 
+    command = [
+        os.environ[ "PYTHON" ],
+        nuitka_main_path,
+        path,
+        "--output-dir=%s" % tmp_dir,
+        "--exe",
+        "--recurse-all"
+    ]
     result = subprocess.call(
-        "%s %s %s --output-dir %s --exe --recurse-all" % (
-            os.environ[ "PYTHON" ],
-            nuitka_main_path,
-            path,
-            tmp_dir
-        ),
-        shell = True
+        command
     )
 
     if result != 0:
@@ -293,31 +318,35 @@ def executePASS3():
 
     shutil.rmtree( build_path )
 
-    print( "OK." )
+    my_print( "OK." )
 
 def executePASS4():
-    print( "PASS 4: Compiling the compiler running from single exe" )
+    my_print( "PASS 4: Compiling the compiler running from single exe" )
 
     exe_path = os.path.join( tmp_dir, "nuitka.exe" )
 
     compileAndCompareWith( exe_path )
 
-    print( "OK." )
+    my_print( "OK." )
 
 def executePASS5():
-    print( "PASS 5: Compiling the compiler 'nuitka' package to a single '.so' file." )
+    my_print( "PASS 5: Compiling the compiler 'nuitka' package to a single '.so' file." )
 
     path = os.path.join( "..", "..", "nuitka" )
 
+    command = [
+        os.environ[ "PYTHON" ],
+        nuitka_main_path,
+        path,
+        "--output-dir=%s" % tmp_dir,
+        "--recurse-all",
+        "--recurse-dir=%s" % path,
+        "--module",
+        path
+
+    ]
     result = subprocess.call(
-        "%s %s %s --output-dir %s --recurse-dir=%s" % (
-            os.environ[ "PYTHON" ],
-            nuitka_main_path,
-            path,
-            tmp_dir,
-            path
-        ),
-        shell = True
+        command
     )
 
     if result != 0:
@@ -326,11 +355,24 @@ def executePASS5():
     os.unlink( os.path.join( tmp_dir, "nuitka.so" ) )
     shutil.rmtree( os.path.join( tmp_dir, "nuitka.build" ) )
 
+cross_compilation = "--windows-target" in os.environ.get( "NUITKA_EXTRA_OPTIONS", "" )
+
 executePASS1()
-executePASS2()
+
+if cross_compilation:
+    my_print( "PASS 2: Skipped for cross-compilation case." )
+else:
+    executePASS2()
 executePASS3()
-executePASS4()
+
+if cross_compilation:
+    my_print( "PASS 4: Skipped for cross-compilation case." )
+else:
+    executePASS4()
 
 shutil.rmtree( "nuitka" )
 
 executePASS5()
+
+os.unlink(os.path.join(tmp_dir, "nuitka.exe" ))
+os.rmdir(tmp_dir)
